/**
 * eeMark
 * http://eeclust.de
 *
 * Copyright (c) 2011-2012, ZIH, TU Dresden, Federal Republic of Germany
 *
 * See the file COPYING in the package base directory for details
 **/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <expec.h>
#include <stdint.h>
#include "energymeter_interface.h"

#define SEPARATOR ";"
#define MAX_TL 600000000 /* maximal length of timeline (10 min) fetched with one request */

int connected = EM_DELAY_CONNECTION;
char * counters[256];
char counter[256], host[256];
int port = 0;
int verbose = 0;

/* call back function that is called by expec_initialize */
void dataheap_init(const char* x) {
	//printf("DEBUG: init %s\n", x);
	connected = EM_CONNECTED;
}

void exit_error(char * errmsg) {
	fprintf(stderr, "DataheapClientLib ERROR: %s\n", errmsg);
	exit(EXIT_FAILURE);
}

int em_connect() {
	//printf("connecting to %s:%d...\n", host, port);
	//expec_initialize(host, port, &dataheap_init, 0); //TODO: blocking initialize? timeout?
	//sleep(1);
	//return connected;
	return EM_DELAY_CONNECTION; /* connect to db after all iterations */
}

void em_init() {
	char *addr=NULL, *tmp=NULL;
	connected = EM_DELAY_CONNECTION;

	//get address from ENV ("host:port")
	addr = getenv("DHLIB_SERVER");
	if(addr==NULL) {
		exit_error("environment variable \"DHLIB_SERVER\" not set.\n");
	}
	tmp=strtok(addr, ":");
	if(tmp!=NULL) {
		strcpy(host, tmp); //copy host part to global variable 'host'
		tmp = strtok(NULL, ":");
		if(tmp==NULL) {
			exit_error("no port given in DHLIB_SERVER. Use format: \"host:port\".\n");
		}
		port = atoi(tmp); //set global variable 'port'
	}
	else {
		exit_error("invalid \"DHLIB_SERVER\".\n");
	}

	//get counter from ENV
	tmp = getenv("DHLIB_COUNTER");
	if(tmp==NULL) {
		exit_error("environment variable \"DHLIB_COUNTER\" not set.\n");
	}
	strcpy(counter, tmp);

	//optional verbose setting
	tmp = getenv("DHLIB_VERBOSE");
	if(tmp!=NULL) {
		verbose = atoi(tmp);
	}
}

void em_close() {
	expec_finalize();
        connected=EM_DELAY_CONNECTION;
}

time_line* make_dh_request(char * counter, uint64_t start, uint64_t stop)
{
	char buf[100];
	int wait,connection_timeout=5;
	memset(buf, 0, 100);

	if (connected==EM_DELAY_CONNECTION) {
		//fprintf(stderr, "DataheapClientLib WARNING: connection to dataheap failed.\n");
		//return 0;
		printf("connecting to %s:%d...\n", host, port);
		expec_initialize(host, port, &dataheap_init, 0);
		wait=0;
		while ((connected!=EM_CONNECTED)&&(wait<=connection_timeout)) {sleep(1);wait++;} //wait for connection
	}

	if (connected!=EM_CONNECTED) {
		fprintf(stderr, "DataheapClientLib WARNING: connection to dataheap failed.\n");
		return 0;
	}

	start /= 1000;
	stop /= 1000;

	if (expec_counterStatus(counter) != 0) //make sure the counter exists, so that the dataheap doesn't quit
	{
		sprintf(buf, "TIMELINE(%llu,%llu,\"%s\", 0)", (unsigned long long)start, (unsigned long long)stop, counter);
		if(verbose>0) {
			printf("DataheapClientLib VERBOSE: requesting timeline for counter %s from %lld to %lld\n", counter, (unsigned long long)start, (unsigned long long)stop);
		}
		return expec_storageRequestTimeLine(buf);
	}
	else
	{
		fprintf(stderr, "DataheapClientLib WARNING: counter %s isn't know on dataheap.\n", counter);
		return 0;
	}


}

double em_get_energy(uint64_t start, uint64_t stop) {
	char * ptr;
	void* dptr;
	double sum = 0.0,t;
	int count = 0, i, j;
	time_line *tl;
	uint64_t valuecount,start_ts,end_ts;

	//tokenize our counter variable into separate counters and save them. this is done beforehand because calls to libexpec functions seem to break strtok
	char ** counters = NULL;
	ptr = strtok(counter, SEPARATOR);

	while(ptr != NULL) {
		int len = strlen(ptr);
		counters = realloc(counters, ++count * sizeof(char*)); //increase counters size
		counters[count-1] = calloc(len+1, sizeof(char)); //allocate char array for new counter
		strncpy(counters[count-1], ptr, len);
		ptr = strtok(NULL, SEPARATOR);
	}
	
	for(j=0; j<count; j++) {
	   end_ts=start;
	   while(end_ts<stop){
		start_ts=end_ts;
		end_ts+=MAX_TL;
		if (end_ts>stop) end_ts=stop;

		tl = make_dh_request(counters[j], start_ts, end_ts);
		if (tl==NULL) {
			free(counters[j]);
			continue;
		}
		valuecount = tl->count;
		dptr = tl->data;

		t = 0.0;
		if(valuecount>0){
			for(i=0; i<valuecount;i++) t += ((double*)dptr)[i*2+1];
			t = t/valuecount; //average watts
		}
		if(verbose>0) printf("DataheapClientLib VERBOSE: %llu element(s), duration = %.3lf ms, average = %.3lf W\n", (unsigned long long)valuecount, (double)(end_ts-start_ts)/1000, t);

		t = t * (end_ts-start_ts); // watts * usecs = microjoule
		sum += t / 1000;     // millijoule
	    }
	    free(counters[j]);
	}
	
	free(counters);
	return sum;
}

double em_get_power(uint64_t start, uint64_t stop) {
	char * ptr;
	void* dptr;
	double sum = 0.0,t;
	int count = 0, i, j;
	time_line *tl;
	uint64_t valuecount,start_ts,end_ts,total;

	//tokenize our counter variable into separate counters and save them. this is done beforehand because calls to libexpec functions seem to break strtok
	char ** counters = NULL;
	ptr = strtok(counter, SEPARATOR);

	while(ptr != NULL) {
		int len = strlen(ptr);
		counters = realloc(counters, ++count * sizeof(char*)); //increase counters size
		counters[count-1] = calloc(len+1, sizeof(char)); //allocate char array for new counter
		strncpy(counters[count-1], ptr, len);
		ptr = strtok(NULL, SEPARATOR);
	}
	
	total=stop-start;
	for(j=0; j<count; j++) {
	   end_ts=start;
	   while(end_ts<stop){
		start_ts=end_ts;
		end_ts+=MAX_TL;
		if (end_ts>stop) end_ts=stop;

		tl = make_dh_request(counters[j], start_ts, end_ts);
		if (tl==NULL) {
			free(counters[j]);
			continue;
		}
		valuecount = tl->count;
		dptr = tl->data;

		t = 0.0;
		if(valuecount>0){
			for(i=0; i<valuecount;i++) t += ((double*)dptr)[i*2+1];
			t = t/valuecount; //average watts
		}
		if(verbose>0) printf("DataheapClientLib VERBOSE: %llu element(s), duration = %.3lf ms, average = %.3lf W\n", (unsigned long long)valuecount, (double)(end_ts-start_ts)/1000, t);

		sum += t*(double)(end_ts-start_ts)/(double)total;
	    }
	    free(counters[j]);	
	}
	free(counters);
	return sum;
}

void em_start_measurement() {
	return;
}
void em_stop_measurement() {
	return;
}

