#ifndef _HAVE_COUNTER_MONITOR_H_
#define _HAVE_COUNTER_MONITOR_H_

// defines to decode the counter status fields
#define STATUS_DATABASE_BITS( X ) (( X>>56 )&0xf )
#define STATUS_DATASOURCE_BITS( X ) (( X>>48 )&0xf )

#include <stdint.h>
#include <socket.h>

/**
 * Counter status types
 */
typedef enum
{
    STATUS_GOOD       = 1,         //!< STATUS_GOOD
    STATUS_FAILED     = 2,         //!< STATUS_FAILED
    STATUS_CONNECTING = 3,         //!< STATUS_CONNECTING
    STATUS_BOOTING    = 4,         //!< STATUS_BOOTING
    STATUS_DOWN       = 5          //!< STATUS_DOWN
} COUNTER_STATUS;

/**
 * IDs for the remote function for the counter
 * listener calls
 */
typedef enum   // remmeber 0 and 1 are reserved
{ FUNCTION_INIT           = 0,
  FUNCTION_RETURN         = 1,
  FUNCTION_LIST_COUNTER   = 4,           //!< FUNCTION_LIST_COUNTER
  FUNCTION_SUBSCRIBE      = 5,           //!< FUNCTION_SUBSCRIBE
  FUNCTION_UNSUBSCRIBE    = 6,           //!< FUNCTION_UNSUBSCRIBE
  FUNCTION_NEW_DATA       = 7,           //!< FUNCTION_NEW_DATA
  FUNCTION_COUNTER_STATUS = 8,           //!< FUNCTION_COUNTER_STATUS
  FUNCTION_DESCRIPTION    = 9,           //!< FUNCTION_COUNTER_STATUS
  FUNCTION_UNIT           = 10,          //!< FUNCTION_COUNTER_STATUS
  FUNCTION_DATA_REQUEST   = 11,          //!< FUNCTION_DATA_REQUEST
  FUNCTION_GET_TIMESTAMP  = 12,          //!< FUNCTION_GET_TIMESTAMP
} COUNTER_MONITOR_FUNCS;

/**
 * IDs for the remote function for the counter
 * source calls
 */
typedef enum                            // remmeber 0 and 1 are reserved
{ FUNCTION_LIST_SOURCES       = 4,      //!< FUNCTION_LIST_SOURCES
  FUNCTION_SET_SOURCE_ID      = 5,      //!< FUNCTION_SET_SOURCE_ID
  FUNCTION_CS_NEW_UINT64_DATA = 6,      //!< FUNCTION_NEW_DATA
  FUNCTION_CS_NEW_DOUBLE_DATA = 7,      //!< FUNCTION_NEW_DATA
  //FUNCTION_GET_TIMESTAMP      = 12,      //!< FUNCTION_GET_TIMESTAMP
} COUNTER_SOURCE_FUNCS;

/** callback type for new data */
typedef void ( *expec_data_callback )( uint64_t dataSourceID,
                                       uint64_t timestamp,
                                       double value );

/** callback type for counterlist */
typedef string_list*( *expec_cl_callback )();

/** callback type for setting the id for a counter */
typedef void ( *expec_sid_callback )( const char* name,
                                      uint64_t    dataSourceID );

/** callback that will be called if set during
 *  thread setup
 */
typedef void ( *expec_init_callback )( const char* );

/** callback to be called upon disconnect event 
 */
typedef void ( *expec_disconnect_callback )( void );

/**
 * initialize library for counter source
 * @param server server IP
 * @param port port number
 * @param counter_list_cb function to retrieve a list of counters
 * @param set_id_cb function to set a counter id
 */
void expec_counter_source_initialize( const char*        server,
                                      int                port,
                                      expec_cl_callback  counter_list_cb,
                                      expec_sid_callback set_id_cb );

/**
 * lists all counters known to the manager
 * @return a string list
 */
string_list *expec_listCounter();

/**
 * subscribes to a one counter to get
 * updates for counter values
 * @param name the name of the counter
 * @param post_mortem whether to have live data or all data in
 *                    one chunk afterwards
 * @return the counter id, 0 on failure
 */
uint64_t expec_subscribe( char* name,
                          char  post_mortem );

/**
 * gets a human readable description for a counter
 * @param name the name of the counter
 * @return a description
 */
char* expec_description( char* name );

/**
 * gets a measument unit for a counter
 * @param name the name of the counter
 * @return a unit
 */
char* expec_unit( char* name );




/**
 * makes a database request (double)
 */
double expec_storageRequestSingle(char * request);
time_line* expec_storageRequestTimeLine(char * request);


/**
 * unsubscribes from getting changes for counter
 * values
 * @param counter_id the id of the counter
 * @return a structure that hold the remote counter data
 */
time_line* expec_unsubscribe( uint64_t counter_id );

/**
 * Get a Timestamp from the counter source for sync purposes
  * @param counter_id the id of the counter
 * @return timestamp in milliseconds
 */
uint64_t expec_getTimestamp( uint64_t counter_id );

/**
 * registers a callback that will be called if there
 * is a change for a counter
 * FIXME: up to now the implementation supports only
 *        one callback
 * @param callback pointer to a callback function
 */
void expec_registerNewDataCallback( expec_data_callback callback );

/**
 * removes a callback from a list of callback that would
 * be called if a counter has an update
 * @param callback pointer to a callback function
 */
void expec_unregisterNewDataCallback( expec_data_callback callback );

/**
 * retrieves the status of a counter from the manager. this
 * includes the information whether the storage is connected
 * and whether the counter source is present. the macros
 * provided at the beginning of this header should be used
 * to examinethe result
 * @param name the counter name
 * @return status bits
 */
uint64_t expec_counterStatus( char* name );

/**
 * initializes the connection to the server and announces
 * the object type on the server side. once this function
 * returned the other functions can be called
 * @param server ip address or dns resolvable host name
 * @param port port number
 * @param init_func a function that will be called from the
 *        thread to say "i am here" (or so), can be NULL
 * @param async 0/1 whether to initialize the async or the blocking
 *              version, 1=blocking 0=async
 */
void expec_initialize( const char*         server,
                       int                 port,
                       expec_init_callback init_func,
                       char                post_mortem );

/**
 * finalizes the interface, returns when the started thread has finished
 */
void expec_finalize();

/**
 * send a new data callback to the server process
 * @param dataSourceID the source id
 * @param timestamp the timestamp
 * @param value the value at this point in time
 */
void expec_newData( uint64_t dataSourceID,
                    uint64_t timestamp,
                    double value );


/**
 * sets a callback to be called at a socket problem, like a 
 * disconnect
 */
void expec_setDisconnetCallback( expec_disconnect_callback newcb );


#endif /* _HAVE_COUNTER_MONITOR_H_ */
