/**
 * eeMark
 * http://eeclust.de
 *
 * Copyright (c) 2011-2012, ZIH, TU Dresden, Federal Republic of Germany
 *
 * See the file COPYING in the package base directory for details
 **/
/*
 * powertracer.c
 *
 * Implements the energymeter_interface for the PowerTracer library
 *
 *
 *  Created on: May 16, 2011
 *  Author: Timo Minartz
 */

#include "energymeter_interface.h"
#include "DBConnector.h"
#include "MetricInterpreter.h"

#include <glib.h>

#define PT_MAX_NODES 12

static gint connected = EM_NOT_CONNECTED;
static gchar **nodes = NULL;
static gint verbose = 0;

void exit_error(gchar * errmsg) {
	g_printerr("PowerTracerClientLib ERROR: %s\n", errmsg);
	exit(EXIT_FAILURE);
}

gint em_connect() {

	if (connected == EM_NOT_CONNECTED) {

		gint ret = db_initConnection(1);

		if (ret == OK) {
			connected = EM_CONNECTED;
			if (verbose)
				g_print(
						"PowerTracerClientLib: Successfully connected to database.\n");
			return connected;
		}

		if (verbose)
			g_print("PowerTracerClientLib: Connection failed.\n");

		return connected;

	}

	if (verbose)
		g_print("PowerTracerClientLib: Already connected.\n");

	return connected;

}

void em_init() {

	gchar *envvar = NULL;

	envvar = getenv("PTLIB_NODES");

	if (envvar == NULL) {
		exit_error("environment variable \"PTLIB_NODES\" not set.\n");
	}

	nodes = g_strsplit(envvar, ":", PT_MAX_NODES);

	//optional verbose setting
	envvar = getenv("PTLIB_VERBOSE");
	if (envvar != NULL) {
		verbose = atoi(envvar);
	}

}

void em_close() {
	db_finishConnection();
	connected = EM_NOT_CONNECTED;

	// TODO: free nodes here?
	//g_strfreev(nodes);
}

double em_get_energy(uint64_t start, uint64_t stop) {

	if (connected==EM_NOT_CONNECTED) {
		g_printerr(
				"PowerTracerClientLib WARNING: connection to PowerTracer failed. Call em_connect() first.\n");
		return -1;
	}

	row_struct * rows;
	double totalEnergy = 0.0;

	for (gint i = 0; i < g_strv_length(nodes); ++i) {

		gchar * node = nodes[i];
		int crows = db_getNodeMetric("power", node, start, stop, &rows);

		if (crows < 0) {

			if (crows == DB_CONNECTION_FAILED)
				g_printerr(
						"PowerTracerClientLib WARNING: Connection to PowerTracer failed.\n");
			else if (crows == DB_NO_SUCH_NODE)
				g_printerr(
						"PowerTracerClientLib WARNING: Node %s not known in database.\n", node);
			else if (crows == DB_QUERY_FAILED)
				g_printerr(
						"PowerTracerClientLib WARNING: Query failed with unknown reason.\n");

			return -2;
		}


		if(crows == 0)
			continue; // no data for interval, rows pointer is not allocated (no need to free it)

		double energy = mi_getStepBasedEnergyConsumptionInMilliJoule(&rows, crows);

		if (verbose)
			printf(
					"PowerTracerClientLib: Node %s - %llu element(s), duration = %.3lf ms, energy = %.3lf mJ\n",
					node, (unsigned long long) crows,
					(double) (stop - start) / 1000, energy);

		free(rows);

		totalEnergy += energy;
	}

	return totalEnergy;
}

double em_get_power(uint64_t start, uint64_t stop) {

	if (connected==EM_NOT_CONNECTED) {
		fprintf(
				stderr,
				"PowerTracerClientLib WARNING: Connection to PowerTracer failed. Call em_connect() first.\n");
		return -1;
	}

	row_struct * rows;
	double meanPower = 0.0;

	int cNodes = g_strv_length(nodes);

	if (cNodes == 0)
		return 0.0;

	for (gint i = 0; i < cNodes; ++i) {

		gchar * node = nodes[i];

		int crows = db_getNodeMetric("power", node, start, stop, &rows);

		if (crows < 0) {

			if (crows == DB_CONNECTION_FAILED)
				g_printerr(
						"PowerTracerClientLib WARNING: Connection to PowerTracer failed.\n");
			else if (crows == DB_NO_SUCH_NODE)
				g_printerr(
						"PowerTracerClientLib WARNING: Node %s not known in database.\n", node);
			else if (crows == DB_QUERY_FAILED)
				g_printerr(
						"PowerTracerClientLib WARNING: Query failed with unknown reason.\n");

			return -2;
		}

		double power = mi_getMeanPowerConsumption(&rows, crows);

		if (verbose)
			printf(
					"PowerTracerClientLib: Node %s - %llu element(s), duration = %.3lf ms, average = %.3lf W\n",
					node, (unsigned long long) crows,
					(double) (stop - start) / 1000, power);

		free(rows);

		meanPower += power;

	}

	return meanPower / cNodes;
}

void em_start_measurement() {
	return;
}
void em_stop_measurement() {
	return;
}

/**
 * compile:
 * gcc powertracer.c -I /sw/DBConnector/include/ -L /sw/DBConnector/lib/ -l dbconnector `pkg-config --cflags --libs glib-2.0` -I src/ -std=c99
 *
 * run:
 * export LD_LIBRARY_PATH=/sw/DBConnector/lib:$LD_LIBRARY_PATH
 * export PTLIB_VERBOSE=1
 * export PTLIB_NODES=amd1:amd2
 */
/*
gint main() {
	// test methods

	em_init();

	if (em_connect() != 1) {
		g_print("Connection to PowerTracer failed.");
		return 1;
	}

	// get actual timestamp
	uint64_t before = getSystemTimestamp();

	// sleep 2 seconds
	g_usleep(2000000);

	// get actual timestamp
	uint64_t after = getSystemTimestamp();

	// get power
	double power = em_get_power(before, after);
	if(power < 0) {
		g_print("Error while fetching mean power.\n");
	} else {
		g_print("Mean Power: %f W\n", power);
	}

	// get energy
	double energy = em_get_energy(before, after);
	if(energy < 0) {
		g_print("Error while fetching energy.\n");
	} else {
		g_print("Energy: %f mJ\n", energy);
	}

	em_close();
}
*/
